package com.taglib.wdjsp.filters;

import javax.servlet.http.HttpSession;
import javax.servlet.http.HttpSessionEvent;
import javax.servlet.http.HttpSessionListener;
import javax.servlet.http.HttpSessionBindingEvent;
import javax.servlet.http.HttpSessionAttributeListener;
import java.util.Date;

public class LoginSessionListener 
  implements HttpSessionListener, HttpSessionAttributeListener {

  public void sessionCreated (HttpSessionEvent event) {
    HttpSession session = event.getSession();
    System.out.println(new Date().toString() + ": session created" 
		       + " (" + session.getId() + ")");
  }
  public void sessionDestroyed (HttpSessionEvent event) {
    HttpSession session = event.getSession();
    System.out.println(new Date().toString() + ": session destroyed" 
		       + " (" + session.getId() + ")");
  }

  public void attributeReplaced (HttpSessionBindingEvent event) {
    /* No-op. */
    dbgSBE("attributeReplaced()", event);
  }
  public void attributeAdded (HttpSessionBindingEvent event) {
    dbgSBE("attributeAdded()", event);
    HttpSession session = event.getSession();
    if (event.getName().equals(UserSessionManager.SESSION_USER_ATTRIBUTE)) {
      UserBean user = (UserBean) event.getValue();
      System.out.println(new Date().toString() 
			 + ": session login for " + user.getUsername()
			 + " (" + session.getId() + ")");
    }
  }
  public void attributeRemoved (HttpSessionBindingEvent event) {
    dbgSBE("attributeRemoved()", event);
    HttpSession session = event.getSession();
    if (event.getName().equals(UserSessionManager.SESSION_USER_ATTRIBUTE)) {
      UserBean user = (UserBean) event.getValue();
      System.out.println(new Date().toString() 
			 + ": session " 
			 + (isTimedOut(session) ? "timeout" : "logout")
			 + " for " + user.getUsername()
			 + " (" + session.getId() + ")");
    }
  }
  private boolean isTimedOut (HttpSession session) {
    try {
      long idle = new Date().getTime() - session.getLastAccessedTime();
      return idle > (session.getMaxInactiveInterval() * 1000);
    }
    catch (IllegalStateException e) { return true; }
  }

  final static private boolean DBG = false;
  private void dbgSBE (String methodName, HttpSessionBindingEvent event) {
    if (DBG) {
      HttpSession session = event.getSession();
      System.out.println("DBG " + methodName 
			 + ": Session " + session.getId() + ", "
			 + event.getName() + " = " + event.getValue());
    }
  }
}
