package com.taglib.wdjsp.filters;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletResponseWrapper;
import javax.servlet.ServletOutputStream;

import java.io.OutputStream;
import java.io.ByteArrayOutputStream;
import java.io.ByteArrayInputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;

import javax.servlet.ServletException;
import java.io.IOException;

public class StringReplaceFilter implements Filter {
  private String from, to;

  public StringReplaceFilter () {}

  public void init (FilterConfig config) throws ServletException {
    from = config.getInitParameter("from");
    if (from == null)
      throw new ServletException("The from init parameter"
				 + " must be specified.");
    to = config.getInitParameter("to");
    if (to == null)
      throw new ServletException("The to init parameter"
				 + " must be specified.");
  }
  public void doFilter (ServletRequest request, ServletResponse response,
			FilterChain chain) 
    throws IOException, ServletException {
    if (request instanceof HttpServletRequest
	&& response instanceof HttpServletResponse) {
      HttpServletRequest req = (HttpServletRequest) request;
      HttpServletResponse resp = (HttpServletResponse) response;
      StringReplaceResponse wrappedResponse =
	new StringReplaceResponse(resp);
      chain.doFilter(request, wrappedResponse);
      transferData(wrappedResponse, resp);
    } else {
      throw new ServletException("Filter only applicable"
				 + " to HTTP and HTTPS requests.");
    }
  }
  public void destroy () {}

  private void transferData (StringReplaceResponse fromResponse,
			     HttpServletResponse toResponse) 
    throws IOException {
    ByteArrayInputStream bais = fromResponse.surrogate.getInputStream();
    BufferedReader reader = 
      new BufferedReader(new InputStreamReader(bais));
    ServletOutputStream original = toResponse.getOutputStream();
    PrintWriter writer =
      new PrintWriter(new OutputStreamWriter(original));
    String line;
    while ((line = reader.readLine()) != null) {
      writer.println(doReplacement(line));
    }
    writer.close();
    reader.close();
  }
  private String doReplacement (String s) {
    int start = s.indexOf(from);
    if (start < 0) {
      return s;
    } else {
      return s.substring(0, start) + to 
	+ doReplacement(s.substring(start + from.length()));
    }
  }

  private class StringReplaceResponse extends HttpServletResponseWrapper {
    MyServletOutputStream surrogate = new MyServletOutputStream();

    private StringReplaceResponse (HttpServletResponse response) {
      super(response);
    }
    public ServletOutputStream getOutputStream ()
      throws java.io.IOException {
      return surrogate;
    }
    public PrintWriter getWriter ()
      throws java.io.IOException {
      return new PrintWriter(new OutputStreamWriter(surrogate));
    }
  }
  private class MyServletOutputStream extends ServletOutputStream {
    private ByteArrayOutputStream baos = new ByteArrayOutputStream(1024);
				
    public void write (int b) { baos.write(b); }
    public void write (byte[] b, int off, int len) { 
      baos.write(b, off, len);
    }

    private ByteArrayInputStream getInputStream () {
      return new ByteArrayInputStream(baos.toByteArray());
    }
  }

  static public void main (String[] args) {
    StringReplaceFilter f = new StringReplaceFilter();
    f.from = "s";
    f.to = "th";
    String original = "This is silly, isn't it?";
    System.out.println("ORIGINAL: " + original);
    System.out.println("REPLACED: " + f.doReplacement(original));
  }
}
