package com.taglib.wdjsp.mut;

import javax.servlet.jsp.tagext.TagLibraryValidator;
import javax.servlet.jsp.tagext.ValidationMessage;
import javax.servlet.jsp.tagext.PageData;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Attr;
import java.io.InputStream;
import java.util.Map;
import java.util.ArrayList;

import java.io.IOException;
import org.xml.sax.SAXException;
import javax.xml.parsers.ParserConfigurationException;

public class NestingTLV extends TagLibraryValidator {
  private String inner, outer;
  private DocumentBuilderFactory factory;

  public NestingTLV () {
    factory = DocumentBuilderFactory.newInstance();
    factory.setValidating(false);
    factory.setNamespaceAware(true);
  }
  public void setInitParameters (Map initParms) {
    super.setInitParameters(initParms);
    inner = (String) initParms.get("inner");
    outer = (String) initParms.get("outer");
  }

  public ValidationMessage[] validate (String prefix, String uri,
				       PageData page) {
    ArrayList messages = new ArrayList();
    if (inner == null) 
      messages.add(new ValidationMessage(null, "No inner tag specified."));
    if (outer == null)
      messages.add(new ValidationMessage(null, "No outer tag specified."));
    if (! messages.isEmpty())
      return (ValidationMessage[]) messages.toArray(new ValidationMessage[0]);
    InputStream in = null;
    DocumentBuilder builder;
    try {
      synchronized (factory) {
	builder = factory.newDocumentBuilder();
      }
      in = page.getInputStream();
      Document doc = builder.parse(in);
      validateDocument(messages, doc, 
		       prefix + ":" + inner, prefix + ":" + outer);
    }
    catch (ParserConfigurationException e) {
      messages.add(new ValidationMessage(null, e.getMessage()));
    }
    catch (SAXException e) {
      messages.add(new ValidationMessage(null, e.getMessage()));
    }
    catch (IOException e) {
      messages.add(new ValidationMessage(null, e.getMessage()));
    }
    finally {
      if (in != null) try { in.close(); } catch (IOException e) {}
    }
    return (ValidationMessage[]) messages.toArray(new ValidationMessage[0]);
  }
  private void validateDocument (ArrayList messages,
				 Document doc, String inner, String outer) {
    NodeList innerNodes = doc.getElementsByTagName(inner);
    int len = innerNodes.getLength();
    String errorString = null;
    for (int i = 0; i < len; ++i) {
      ValidationMessage msg =
	validateInnerNode(innerNodes.item(i), inner, outer);
      if (msg != null) messages.add(msg);
    }
  }
  private ValidationMessage validateInnerNode (Node innerNode,
					       String innerName,
					       String outerName) {
    Node parent = findAncestor(innerNode, outerName);
    if (parent == null) {
      NamedNodeMap atts = innerNode.getAttributes();
      Attr idAttr = (Attr) atts.getNamedItem("jsp:id");
      String id = (idAttr == null) ? null : idAttr.getValue();
      return new ValidationMessage(id,
				   "Improper nesting of " + innerName 
				   + " tag: must appear in body of "
				   + outerName + " tag.");
    } else {
      return null;
    }
  }
  private Node findAncestor (Node child, String named) {
    Node parent = child.getParentNode();
    if (parent == null) {
      return null;
    } 
    String parentName = parent.getNodeName();
    if (parentName.equals(named)) {
      return parent;
    } else {
      return findAncestor(parent, named);
    }
  }
}
