package com.taglib.wdjsp.mut;

import javax.servlet.jsp.tagext.TagLibraryValidator;
import javax.servlet.jsp.tagext.ValidationMessage;
import javax.servlet.jsp.tagext.PageData;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.SAXParser;
import org.xml.sax.Attributes;
import org.xml.sax.helpers.DefaultHandler;
import java.io.InputStream;
import java.util.Map;
import java.util.ArrayList;

import java.io.IOException;
import org.xml.sax.SAXException;
import javax.xml.parsers.ParserConfigurationException;

/**
 *
 */
public class ScriptFreeTLV extends TagLibraryValidator {
  private boolean allowDeclarations = false;
  private boolean allowScriptlets = false;
  private boolean allowExpressions = false;
  private boolean allowRTExpressions = false;
  private SAXParserFactory factory;

  public ScriptFreeTLV () {
    factory = SAXParserFactory.newInstance();
    factory.setValidating(false);
    factory.setNamespaceAware(true);
  }
  public void setInitParameters (Map initParms) {
    super.setInitParameters(initParms);
    String declarationsParm = (String) initParms.get("allowDeclarations");
    String scriptletsParm = (String) initParms.get("allowScriptlets");
    String expressionsParm = (String) initParms.get("allowExpressions");
    String rtExpressionsParm = (String) initParms.get("allowRTExpressions");

    allowDeclarations = "true".equalsIgnoreCase(declarationsParm);
    allowScriptlets = "true".equalsIgnoreCase(scriptletsParm);
    allowExpressions = "true".equalsIgnoreCase(expressionsParm);
    allowRTExpressions = "true".equalsIgnoreCase(rtExpressionsParm);
  }

  public ValidationMessage[] validate (String prefix,
				       String uri, PageData page) {
    ArrayList msgs = new ArrayList();
    InputStream in = null;
    SAXParser parser;
    MyContentHandler handler = new MyContentHandler(msgs);
    try {
      synchronized (factory) {
	parser = factory.newSAXParser();
      }
      in = page.getInputStream();
      parser.parse(in, handler);
    }
    catch (ParserConfigurationException e) {
      addMessage(msgs, null, e.getMessage());
    }
    catch (SAXException e) {
      addMessage(msgs, null, e.getMessage());
    }
    catch (IOException e) {
      addMessage(msgs, null, e.getMessage());
    }
    finally {
      if (in != null) try { in.close(); } catch (IOException e) {}
    }
    String results = handler.reportResults();
    if (results != null) addMessage(msgs, null, results, 0);
    return (ValidationMessage[]) msgs.toArray(new ValidationMessage[0]);
  }

  private void addMessage (ArrayList messages, String jspId, String msg,
			   int index) {
    ValidationMessage vMessage = new ValidationMessage(jspId, msg);
    if (index < 0) {
      messages.add(vMessage);
    } else {
      messages.add(index, vMessage);
    }
  }
  private void addMessage (ArrayList messages, String jspId, String msg) {
    addMessage(messages, jspId, msg, -1);
  }

  private class MyContentHandler extends DefaultHandler {
    private ArrayList messages;
    private int declarationCount = 0;
    private int scriptletCount = 0;
    private int expressionCount = 0;
    private int rtExpressionCount = 0;

    MyContentHandler (ArrayList messages) { this.messages = messages; }

    public void startElement (String namespaceUri, String localName,
			      String qualifiedName, Attributes atts) {
      if ((! allowDeclarations)
	  && qualifiedName.equals("jsp:declaration")) {
	++declarationCount;
	addMessage(messages, getJspId(atts), "Declaration found.");
      } else if ((! allowScriptlets)
		 && qualifiedName.equals("jsp:scriptlet")) {
	++scriptletCount;
	addMessage(messages, getJspId(atts), "Scriptlet found.");
      } else if ((! allowExpressions)
		 && qualifiedName.equals("jsp:expression")) {
	++expressionCount;
	addMessage(messages, getJspId(atts), "Expression found.");
      }
      if (! allowRTExpressions) countRTExpressions(atts);
    }
    public void countRTExpressions (Attributes atts) {
      int stop = atts.getLength();
      for (int i = 0; i < stop; ++i) {
	String attval = atts.getValue(i);
	if (attval.startsWith("%=") && attval.endsWith("%")) {
	  ++rtExpressionCount;
	  addMessage(messages, getJspId(atts),
		     "Request-time attribute value expression found.");
	}
      }
    }
    private String getJspId (Attributes atts) {
      return atts.getValue("jsp:id");
    }

    public String reportResults () {
      if (declarationCount + scriptletCount + expressionCount > 0) {
	StringBuffer results = new StringBuffer("JSP page contains ");
	boolean first = true;
	if (declarationCount > 0) {
	  results.append(Integer.toString(declarationCount));
	  results.append(" declaration");
	  if (declarationCount > 1) results.append('s');
	  first = false;
	}
	if (scriptletCount > 0) {
	  if (! first) results.append(", ");
	  results.append(Integer.toString(scriptletCount));
	  results.append(" scriptlet");
	  if (scriptletCount > 1) results.append('s');
	  first = false;
	}
	if (expressionCount > 0) {
	  if (! first) results.append(", ");
	  results.append(Integer.toString(expressionCount));
	  results.append(" expression");
	  if (expressionCount > 1) results.append('s');
          first = false;
	}
	if (rtExpressionCount > 0) {
	  if (! first) results.append(", ");
	  results.append(Integer.toString(rtExpressionCount));
	  results.append(" request-time attribute value");
	  if (rtExpressionCount > 1) results.append('s');
	}
	results.append(".");
	return results.toString();
      } else {
	return null;
      }
    }
  }
}
